import {Component, ElementRef, ViewChild} from '@angular/core';
import axios from "axios";
import {DevelopmentStore} from "../../store/DevelopmentStore";
import {FormsModule} from "@angular/forms";
import {NgIf} from "@angular/common";

@Component({
  selector: 'app-download',
  standalone: true,
  imports: [
    FormsModule,
    NgIf
  ],
  templateUrl: './download.component.html',
  styleUrl: './download.component.scss'
})
export class DownloadComponent {

  @ViewChild('download_not_possible') download_not_possible: ElementRef<HTMLDialogElement> | undefined;

  inputFileId: string = "";
  fileId: string = "";
  filePassword: string = "";
  downloadInfo: DownloadInfo | null = null;

  waitingForPassword: boolean = false;


  constructor(private developmentStore: DevelopmentStore) {
    this.speedTest();
  }

  requestDownload() {

    console.log(this.inputFileId);
    this.fileId = this.inputFileId; // TODO: Implement link extraction logic

    this.getDownloadInfo();
  }

  processDownloadInfo() {
    if(!this.downloadInfo?.passwordProtected && this.downloadInfo?.downloadable) {
      console.log("Proceeding with download");
      this.downloadFile();
      return;
    }
    else if(!this.downloadInfo?.downloadable) {
      this.download_not_possible?.nativeElement.showModal();
      return;
    }
    else if(this.downloadInfo?.passwordProtected) {
      console.log("Password protected");
    }
  }

  private speedTest() {
    const start = new Date().getTime(); // Start timer

    axios({
      method: 'get',
      url: this.developmentStore.getBaseUrl() + 'api/v1/speed-test',
      responseType: 'arraybuffer',
      headers: {
        'Access-Control-Allow-Origin': '*', // Allow CORS
      }
    })
      .then(response => {
        const end = new Date().getTime(); // End timer
        const duration = (end - start) / 1000; // Convert ms to seconds
        const bitsLoaded = response.data.byteLength * 8;
        const speedBps = bitsLoaded / duration;
        const speedKbps = speedBps / 1024;
        const speedMbps = speedKbps / 1024;
        console.log(`Download speed: ${speedMbps.toFixed(2)} Mbps`);
      })
      .catch(error => {
        console.error('Error during download test:', error);
      });
  }

  private getDownloadInfo() {
    axios({
      method: 'get',
      url: this.developmentStore.getBaseUrl() + 'api/v1/download-info?fileId=' + this.fileId,
      responseType: 'json',
      headers: {
        'Access-Control-Allow-Origin': '*', // Allow CORS
      }
    })
    .then(response => {
      this.downloadInfo = response.data;
      console.log(response.data);
      this.processDownloadInfo();
    })
    .catch(error => {
      console.error('Error during download info request:', error);
    });
  }

  private downloadFile() {
    axios({
      method: 'get',
      url: this.developmentStore.getBaseUrl() + 'api/v1/download?fileId=' + this.fileId,
      responseType: 'arraybuffer',
      headers: {
        'Access-Control-Allow-Origin': '*', // Allow CORS
      },
      onDownloadProgress: function(progressEvent) {
        // Calculate the percentage of download completed
        if(progressEvent.total) {
          const percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
          console.log(percentCompleted + '%'); // Log the percentage or update any progress UI component
        }
      }
    })
    .then(response => {
      const contentDisposition = response.headers['content-disposition'];
      let filename = "default_filename"; // Default filename in case parsing fails

      if (contentDisposition) {
        const filenameRegex = /filename="?([^"]+)"?/;
        const matches = contentDisposition.match(filenameRegex);
        if (matches && matches[1]) {
          filename = matches[1];
        }
      }

      const blob = new Blob([response.data], {type: 'application/octet-stream'});
      const url = window.URL.createObjectURL(blob);
      const a = document.createElement('a');
      a.style.display = 'none';
      a.href = url;
      a.download = filename; // You can specify a filename here
      document.body.appendChild(a);
      a.click();

      // Clean up by revoking the Blob URL and removing the temporary anchor
      window.URL.revokeObjectURL(url);
      document.body.removeChild(a);
    })
    .catch(error => {
      console.error('Error during download request:', error);
    });
  }
}
interface DownloadInfo {
  downloadable: boolean;
  passwordProtected: boolean;
  singleDownload: boolean;
  fileId: string;
}
